#include "usart.h"
#include "stdio.h"
#include "string.h"
#include "ch32v20x.h"
#include "systick_timer.h"

#define RX_BUFF_SIZE 256
#define TX_BUFF_SIZE 256

struct UART
{
    unsigned char tx_buff[TX_BUFF_SIZE];//Ҫ͵
    unsigned char rx_buff[RX_BUFF_SIZE];//յ
    unsigned char rx_len;               //յ֡
    unsigned char received;             //Ƿյ
    unsigned char dma_tc_flag;          //dmaɱ־λ
}uart;

void Usart_Init()
{
    USART_DeInit(USART2);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    DMA_InitTypeDef DMA_InitStructure = {0};
    GPIO_InitTypeDef  GPIO_InitStructure = {0};
    USART_InitTypeDef USART_InitStructure = {0};
    NVIC_InitTypeDef  NVIC_InitStructure = {0};

    USART_InitStructure.USART_BaudRate = 19200;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;

    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    USART_Init(USART2, &USART_InitStructure);
    USART_ITConfig(USART2, USART_IT_IDLE, ENABLE);
    USART_Cmd(USART2, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
    NVIC_Init(&NVIC_InitStructure);

    DMA_DeInit(DMA1_Channel6);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)(&USART2->DATAR);
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)uart.rx_buff;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize = RX_BUFF_SIZE;
    DMA_Init(DMA1_Channel6, &DMA_InitStructure);
    DMA_Cmd(DMA1_Channel6, ENABLE);

    DMA_DeInit(DMA1_Channel7);
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)uart.tx_buff;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = TX_BUFF_SIZE;
    DMA_Init(DMA1_Channel7, &DMA_InitStructure);

    USART_DMACmd (USART2, USART_DMAReq_Rx, ENABLE);
    USART_DMACmd (USART2, USART_DMAReq_Tx, ENABLE);

    //485ʹ
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    RS485_RX;
}

void Uart_Send(unsigned char *data, unsigned int len)
{
    DMA_Cmd(DMA1_Channel7, DISABLE);
    memcpy(uart.tx_buff, data, len);
    DMA_SetCurrDataCounter(DMA1_Channel7, len);
    RS485_TX;
    DMA_Cmd(DMA1_Channel7, ENABLE);
}

//Ƿյһ֡
unsigned char Uart_Received()
{
    return uart.received;
}

//ؽյݳ
unsigned char Uart_Received_Size()
{
    return uart.rx_len;
}

//flagÿյһ֡ݺҪ治
void Uart_Clear_Flag()
{
    uart.received = 0;
    uart.rx_len = 0;
}

//ȡ
unsigned char *Uart_Read()
{
    return uart.rx_buff;
}

void Uart_Set_BaudRate(unsigned int baudrate)
{
    USART_InitTypeDef USART_InitStructure = {0};
    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
    USART_Init(USART2, &USART_InitStructure);
}

//жϲ
void Uart_IDLE_Callback()
{
    DMA_Cmd(DMA1_Channel6, DISABLE);
    uart.rx_len = RX_BUFF_SIZE - DMA_GetCurrDataCounter(DMA1_Channel6);
    DMA_SetCurrDataCounter(DMA1_Channel6, RX_BUFF_SIZE);
    DMA_Cmd(DMA1_Channel6, ENABLE);
    uart.received = 1;
}

void USART2_IRQHandler()
{
    if(USART_GetITStatus(USART2, USART_IT_IDLE) != RESET)
    {
        Uart_IDLE_Callback();
        USART2->STATR;
        USART2->DATAR;
    }
}

unsigned char Uart_TC_Flag()
{
    return DMA_GetFlagStatus(DMA1_FLAG_TC7);
}

void Uart_Clear_TC_Flag()
{
    DMA_ClearFlag(DMA1_FLAG_TC7);
}

__attribute__((used)) int _write(int fd, char *buf, int size)
{
    int i;
    for(i=0; i<size; i++)
    {
        while(USART_GetFlagStatus(USART2, USART_FLAG_TC) == RESET);
        USART_SendData(USART2, *buf++);
    }
    return size;
}

//void DMA1_4_IRQHandler()
//{
//    if(DMA_GetITStatus(DMA1_IT_TC4) != RESET)
//    {
//        uart.tc_flag = 1;
////        DMA_ClearITPendingBit(DMA1_IT_TC4);
////        DMA_ClearITPendingBit(DMA1_IT_GL4);
//        DMA_ClearFlag(DMA1_IT_TC4);
//        printf("ttt\r\n");
//    }
//}
